﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media.Media3D;
using SciChart.Charting.ChartModifiers;
using SciChart.Charting3D;
using SciChart.Charting3D.Model;
using SciChart.Charting3D.Model.ChartData;
using SciChart.Charting3D.Modifiers;
using SciChart.Charting3D.Primitives;
using SciChart.Charting3D.RenderableSeries;
using SciChart.Core.Extensions;
using SciChart.Core.Utility.Mouse;
using SciChart.Drawing.Utility;

namespace DataPointSelectionModifier
{
  internal class DataPointSelectionModifier3D : VertexSelectionModifier3D
  {
    private readonly Dictionary<uint, HashSet<ulong>> mSelectedVertices = new Dictionary<uint, HashSet<ulong>>();

    protected override SelectionMode GetSelectionMode(MouseModifier modifierKey, bool isAreaSelection)
    {
      SelectionMode selectionMode = base.GetSelectionMode(modifierKey, isAreaSelection);

      return selectionMode;
    }

    public override void OnModifierMouseUp(ModifierMouseArgs mouseArgs)
    {
      if (mouseArgs.Modifier == MouseModifier.Ctrl)
      {
        base.OnModifierMouseUp(mouseArgs);
        return;
      }

      Point mousePoint = mouseArgs.MousePoint;
      var translatedPoint = this.RootGrid.TranslatePoint(mousePoint, this.ParentSurface.ModifierSurface);
      List<EntityVertexId>? entityVertexIds = Viewport3D.PickScene(mousePoint.ToRectCenter(30));
      
      //reset all inScene
      Viewport3D.RootEntity.VisitEntities(entity =>
      {
        if (entity is Points3DSceneEntity pointsEntity)
        {
          var seriesInfo = pointsEntity.ToSeriesInfo(new HitTestInfo3D());
          if(seriesInfo?.RenderableSeries.DataSeries is IXyzDataSeries3D dataSeries)
          {
            foreach (var pointMetadata3DEx in dataSeries.GetInSceneMetadata(true))
            {
              pointMetadata3DEx.IsInScene = false;
              pointMetadata3DEx.IsSelected = false;
            }
          }
        }
      });
      
      double distance = double.NaN;
      SeriesInfo3D? nearestSeriesInfo3D = null;
      if (entityVertexIds.Count > 0)
      {
        foreach (var entityVertexId in entityVertexIds)
        {
          Viewport3D.RootEntity.VisitEntities(entity =>
          {
            if (entity is Points3DSceneEntity pointsEntity)
            {
              if (entity.EntityId == entityVertexId.EntityId)
              {
                HitTestInfo3D hitTestInfo = new HitTestInfo3D()
                  { IsHit = true, EntityId = entityVertexId.EntityId, VertexId = entityVertexId.VertexId };
                SeriesInfo3D? seriesInfo = pointsEntity.ToSeriesInfo(hitTestInfo);
                
                //Vector3D hitVertex = seriesInfo.HitVertex;
                Vector3 vertex = seriesInfo.HitVertexCoords;
                Point point1 = ParentSurface.Camera.WorldToScreenSpace(vertex);
                Point point2 = ParentSurface.Camera.LocalCoordinateToScreenSpace(vertex);

                if (seriesInfo.PointMetadata is PointMetadata3DEx locatedMetadata)
                {
                  locatedMetadata.IsInScene = true;
                  locatedMetadata.ScreenPoint1 = point1;
                  locatedMetadata.ScreenPoint2 = point2;
                  locatedMetadata.LastMousePoint = mousePoint;
                  locatedMetadata.MousePointTranslated = translatedPoint;
                }

                //determine point closest to mouse point
                double newDistance = PointUtil.Distance(translatedPoint, point1);
                if (distance.IsNaN() || newDistance < distance)
                {
                  distance = newDistance;
                  nearestSeriesInfo3D = seriesInfo;
                }
              }
            }
          });
        }

        IXyzDataSeries3D? selectedDataSeries = nearestSeriesInfo3D?.RenderableSeries?.DataSeries as IXyzDataSeries3D;
        if (selectedDataSeries == null)
        {
          return;
        }

        //if selection or focusing not possible - return
        if (!(nearestSeriesInfo3D?.PointMetadata is PointMetadata3DEx currentHitMetadata))
        {
          return;
        }

        currentHitMetadata.IsSelected = !currentHitMetadata.IsSelected;
      }
    }
  }
}
