﻿using SciChart.Charting3D.Model;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DataPointSelectionModifier
{
  internal static class IXyzDataSeries3DExtension
  {
    private const string MetaDataPropertyName = nameof(XyzDataSeries3D<int, int, int>.WValues);

    internal static IEnumerable<PointMetadata3DEx> GetMetadata(this IXyzDataSeries3D dataSeries)
    {
      var result = new List<PointMetadata3DEx>();
      //check if selectedDataSeries is XyzDataSeries3D (because of Property WValues, that contains Point3DMetaData)
      if (dataSeries.GetType().GetGenericTypeDefinition() == typeof(XyzDataSeries3D<,,>))
      {
        result = (dataSeries.InvokeProperty<IList>(MetaDataPropertyName)?.OfType<PointMetadata3DEx>().ToList() ?? new List<PointMetadata3DEx>());
      }

      return result;
    }

    internal static IEnumerable<PointMetadata3DEx> GetInSceneMetadata(this IXyzDataSeries3D dataSeries, bool isInScene)
    {
      return dataSeries.GetMetadata().Where(p => p.IsInScene == isInScene);
    }

    internal static IEnumerable<PointMetadata3DEx> GetSelectedMetadata(this IXyzDataSeries3D dataSeries)
    {
      return dataSeries.GetMetadata().Where(p => p.IsSelected);
    }

    public static T? InvokeProperty<T>(this object? invoker, string propertyName) where T : class
    {
      if (string.IsNullOrEmpty(propertyName) || invoker == null)
      {
        return default;
      }

      var propertyInfo = invoker.GetType().GetProperty(propertyName);

      if (propertyInfo == null)
      {
        Debug.Assert(false, $"object of type {invoker.GetType()} does not contain property:  {propertyName}");
        return default;
      }

      return propertyInfo.GetValue(invoker) as T;

    }
  }
}
